/**
 * Compiler for the css.
 * @constructor TheCssCompiler
 * @param {object} config
 * @param {string} config.prefix - Css prefix.
 */

"use strict";

var argx = require('argx'),
    arrayfilter = require('arrayfilter'),
    async = require('async'),
    css = require('css');

/** @lends TheCssCompiler */
function TheCssCompiler(config) {
    var s = this;
}

TheCssCompiler.prototype = {
    /**
     * Compile css string.
     * @param {string} source - Css string.
     * @param {object} [options] - Optional settings.
     * @param {function} callback - Callback when done.
     */
    compileCss: function (source, options, callback) {
        var args = argx(arguments);
        source = args.shift('string');
        callback = args.pop('function') || argx.noop;
        options = args.pop('object') || {};
        var prefix = options.prefix;
        var data = css.parse(source, options);
        async.waterfall([
            function (callback) {
                var stylesheet = data['stylesheet'];
                callback(null, stylesheet['rules']);
            },
            function (rules, callback) {
                var stylesheet = {
                    rules: rules.map(function (rule) {
                        return {
                            type: rule.type,
                            selectors: rule.selectors.map(function (selector) {
                                return [
                                    prefix,
                                    selector
                                ]
                                    .filter(arrayfilter.emptyReject(), [])
                                    .join(' ')
                            }),
                            declarations: rule.declarations
                        }
                    })
                };
                callback(null, css.stringify({
                    stylesheet: stylesheet
                }));
            },
            function (string, callback) {
                callback(null, string);
            }
        ], callback);
    }
};

module.exports = TheCssCompiler;
